#include <QtWidgets/QApplication>
#include <osgUtil/SmoothingVisitor>
#include <osgUtil/Optimizer>
#include <osgDB/ReadFile>
#include <PoLAR/Viewer.h>
#include <PoLAR/Image.h>

#include "MyVierwerOverPainted.h"

int main(int argc, char **argv)
{
    int width = 640;
    int height = 480;
    char mode = 'i';
    std::string backgroundImagePath = "../resources/desk.jpg";
    std::string projectionPath = "../resources/projection.proj";
    std::string coffeeMugPath = "../resources/coffeeMug.obj";

    QApplication app(argc, argv);
    MyViewerOverPainted viewer;
    osgUtil::SmoothingVisitor smooth;
    osgUtil::Optimizer optimizer;

    PoLAR::Light* light;
    osg::ref_ptr<PoLAR::Image_uc> backgroundImage;
    osg::ref_ptr<PoLAR::Object3D> coffeeMug;
    osg::ref_ptr<PoLAR::Object3D> ground;

    if (argc != 1)
    {
        if (*argv[1] == 'v')
        {
            mode = 'v';
        }
        else if (*argv[1] != 'i' || argc != 2)
        {
            std::cerr << "Syntax: " << argv[0] << " [<mode> = i(IMAGE) or v(VIDEO), default(IMAGE)]" << std::endl;
            exit(0);
        }
    }

    if (mode == 'i')
    {
        //Background image
        if (!PoLAR::Util::fileExists(backgroundImagePath))
        {
            std::cerr << "Background image not found" << std::endl;
            exit(0);
        }
        backgroundImage = new PoLAR::Image_uc(backgroundImagePath, true);
        viewer.resize(width, height);
        viewer.setBgImage(backgroundImage);
        viewer.bgImageOn();
        viewer.startEditImageSlot();

        //Projection matrix
        if (!PoLAR::Util::fileExists(projectionPath))
        {
            std::cerr << "Projection matrix not found" << std::endl;
            exit(0);
        }
        osg::Matrixd projectionMatrix = PoLAR::Util::readProjectionMatrix(projectionPath.c_str());
        viewer.setProjection(projectionMatrix, PoLAR::Viewer2D3D::VISION);

        //3D objects
        if (!PoLAR::Util::fileExists(coffeeMugPath))
        {
            std::cerr << "3D objects not found" << std::endl;
            exit(0);
        }
        osg::ref_ptr<osg::Node> loadedCoffeeMug = osgDB::readNodeFile(coffeeMugPath);
        optimizer.optimize(loadedCoffeeMug.get());
        loadedCoffeeMug->accept(smooth);
        coffeeMug = new PoLAR::Object3D(loadedCoffeeMug.get(), true, true);
        coffeeMug->scale(0.1);
        coffeeMug->translate(0, 40, 0);
        coffeeMug->rotate(M_PI, 1, 0, 0);
        coffeeMug->rotate(0, 0, 1, 0);
        coffeeMug->rotate(-M_PI_4, 0, 0, 1);

        //Ground
        ground = PoLAR::Object3D::createGround(40.0f, 70.0f);
        ground->translate(25, 35, 25);
        ground->rotate(0, 1, 0, 0);
        ground->rotate(0, 0, 1, 0);
        ground->rotate(M_PI/6, 0, 0, 1);
        ground->setPhantomOn();

        //Lights
        light = viewer.addLightSource(-4, 55, -20, false, true);
        light->setViewable(true);

        viewer.addObject3D(coffeeMug.get());
        viewer.addObject3D(ground.get());
        viewer.setTrackNode(coffeeMug.get());
        viewer.center();
        viewer.show();
    }
    else
    {
        //TODO : Video mode
        std::cerr << "Video mode isn't implemented yet" << std::endl;
        exit(0);
    }

    app.connect(&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()));

    return app.exec();
}